@@ -1,33 +1,42 @@
 # Priors for all the hyper-parameters
-check_settings <- function(settings, gp_type, x, y) {
+check_settings <- function(settings, gp_type, x, y, vdims = NULL) {
   
   dab <- laGP::darg(NULL, x)$ab
-  # gab <- laGP::garg(NULL, y)$ab
   
+  if (is.null(settings$a$tau2_y)) settings$a$tau2_y <- 10/2
+  if (is.null(settings$b$tau2_y)) settings$b$tau2_y <- 4/2
+
   # priors for proposal distribution for Length-scales
   if (is.null(settings$l)) settings$l <- 1
   if (is.null(settings$u)) settings$u <- 2
   
   # Priors for ls - Latent layer (Lambdas)
   if(gp_type == "hetgp"){
-    if (is.null(settings$alpha$theta_lam)) settings$alpha$theta_lam <- dab[1]
-    if (is.null(settings$beta$theta_lam)) settings$beta$theta_lam <- dab[2]
+    
+    if (is.null(settings$a$tau2_lam)) settings$a$tau2_lam <- 10/2
+    if (is.null(settings$b$tau2_lam)) settings$b$tau2_lam <- 4/2
+    
+    if (is.null(settings$alpha$theta_lam)) settings$alpha$theta_lam <- dab[1] * 2
+    if (is.null(settings$beta$theta_lam)) settings$beta$theta_lam <- dab[2]/2
+    
+  }else if(gp_type == "hetgpv"){
+    
+    if (is.null(settings$a$tau2_lam)) settings$a$tau2_lam <- 10/2
+    if (is.null(settings$b$tau2_lam)) settings$b$tau2_lam <- 4/2
+    
+    dab <- laGP::darg(NULL, x[, vdims, drop = FALSE])$ab
+    if (is.null(settings$alpha$theta_lam)) settings$alpha$theta_lam <- dab[1] * 2
+    if (is.null(settings$beta$theta_lam)) settings$beta$theta_lam <- dab[2]/2
+    
   }
-  #else if(gp_type == "homgp"){ # initialize nugget hypers 
+  
   if (is.null(settings$alpha$g)) settings$alpha$g <- 1.5
   if (is.null(settings$beta$g)) settings$beta$g <- 4
-  #}
   
   # Priors for LS - y
   if (is.null(settings$alpha$theta_y)) settings$alpha$theta_y <- dab[1]
   if (is.null(settings$beta$theta_y)) settings$beta$theta_y <- dab[2]
   
-  if (is.null(settings$a$tau2_y)) settings$a$tau2_y <- 10/2
-  if (is.null(settings$b$tau2_y)) settings$b$tau2_y <- 4/2
-  
-  if (is.null(settings$a$tau2_lam)) settings$a$tau2_lam <- 10/2
-  if (is.null(settings$b$tau2_lam)) settings$b$tau2_lam <- 4/2
-  
   return(settings)
 }
 
@@ -48,20 +57,15 @@ check_initialization <- function(reps, initial, n, D, sep, v = NULL, vec = FALSE
       warning("cannot specify scale and use prof ll; will not infer tau2 lam")
       initial$inner <- initial$inner_tau2 <- FALSE # for Lam layer
     }
-  }else
+  }else{
     initial$inner <- initial$inner_tau2  <- FALSE # for Lam layer
+  }
   
   if(verb) print("obtaining initialization")
-  
   if(stratergy == "default"){
-    # if(ncol(reps$X0) > 1 || nrow(reps$X0) <= 1000)
       init <- inits_bhetgp(reps, v = v, vec = vec)
-    # else{
-    #   warning("vec = T but dim(x) = 1. SVec needs dim(x) > 2. Setting stratergy = flat")
-    #   stratergy = "flat"
-    # }  
   }
-  if (stratergy == "flat")
+  if (stratergy == "flat") init <- inits_flat(reps)
   
   if(verb) print("checking specifications")
   if(is.null(initial$mean_y)) initial$mean_y <- mean(reps$Z)
@@ -76,7 +80,7 @@ check_initialization <- function(reps, initial, n, D, sep, v = NULL, vec = FALSE
   if(is.null(initial$noise)) initial$noise = FALSE
 
   # nugget for lam layer numeric stability]
-  if (is.null(initial$g)) initial$g <- 1e-5 
+  if (is.null(initial$g)) initial$g <- 1e-6
   
   if(is.null(initial$theta_lam)) {
     initial$theta_lam <- ifel(sep, init$tg, mean(init$tg))
@@ -202,22 +206,33 @@ check_ordering <- function(ordering, n) {
   return(NULL)
 }
 
-check_scale <- function(x, v, vec = FALSE, priors, init){
+check_scale <- function(x, v, vec = FALSE, priors, init, vdims = FALSE, vd = NULL){
   
-  N <- length(init$llam)
-  count <- 0
-  init$tau2_lam <- 1000 # arbitrary
-  while(init$tau2_lam > 10){
-    init$theta_lam <- init$theta_lam/10
+  if(init$prof_ll_lam){
+    N <- length(init$llam)
+    count <- 0
+    init$tau2_lam <- 1000 # arbitrary
+    init$theta_lam <- init$theta_lam * 10 # start with more so it scales to orig
+    while(init$tau2_lam > 150){
+      # print("adjusting scale")
+      if(count %% 20) init$g <- init$g * 1.05
+      init$theta_lam <- init$theta_lam/10
+      obj <- precalc(x, out = NULL, outs2 = NULL, A = NULL, v, vec, priors, init, 
+                     latent = TRUE, vdims = vdims)
+      init$tau2_lam <- obj$tau2
+      init$llik_lam <- obj$llik
+      count <- count + 1
+    }
+  }else{
     obj <- precalc(x, out = NULL, outs2 = NULL, A = NULL, v, vec, priors, init, 
-                   latent = TRUE)
-    init$tau2_lam <- obj$tau2
+                   latent = TRUE, vdims = vdims)
+    init$tau2_lam <- init$scale_lam
     init$llik_lam <- obj$llik
-    count <- count + 1
-    if(count == 10) init$g <- init$g * 10
   }
-  
-  if(init$theta_check == TRUE & any(init$theta_lam < init$theta_y)) init$theta_y = init$theta_lam
+
+  if(is.null(vd)) vd <- 1:length(init$theta_y)
+  if(init$theta_check == TRUE & any(init$theta_lam < init$theta_y[vd])) 
+    init$theta_y[vd] = init$theta_lam
   
   return(init)
 }
@@ -226,48 +241,59 @@ check_inits_vdims <- function(reps, initial, n, D, sep, vdims = NULL,
                               nlam, reps_vdims, v = NULL, stratergy, 
                               vec, verb = TRUE) {
   
+  
+  # first run inits() and then assign accordingly if any thing not specified?
+  if(is.null(initial$prof_ll_lam)){
+    initial$prof_ll_lam <- TRUE
+    initial$inner <- TRUE # for Lam layer
+    initial$inner_tau2 <- TRUE
+    initial$scale_lam <- 1
+  } 
+  
+  if(initial$prof_ll_lam){
+    if(initial$scale_lam != 1){
+      warning("cannot specify scale and use prof ll; will not infer tau2 lam")
+      initial$inner <- initial$inner_tau2 <- FALSE # for Lam layer
+    }
+  }else{
+    if(is.null(initial$scale_lam)) initial$scale_lam <- 1
+    initial$tau2_lam <- initial$scale_lam
+    initial$inner <- initial$inner_tau2  <- FALSE # for Lam layer
+  }
+  
   if(verb) print("obtaining initialization")
+  
   if(stratergy == "default"){
-    init <- inits_vdims(reps, reps_vdims, v = v, vec = vec)
+    init <- inits_vdims(reps, reps_vdims, v = v, vec = vec, vdims = vdims)
   }
   if (stratergy == "flat"){
-    out <- list(reps = reps, ty = rep(1, D), tg = rep(2, length(vdims)), 
+    init <- list(reps = reps, ty = rep(1, D), tg = rep(2, length(vdims)), 
                 mean0 = 0, scale0 = 1, llam_w = rep(log(var(reps$Z) * 0.1), nlam), 
-                mu_y = 0, scale = 1, g = 1e-5)
+                mu_y = 0, scale = 1, g = 1e-6)
   }
     
-  
   if(verb) print("checking specifications")
+  
   if(is.null(initial$mean_y)) initial$mean_y <- mean(reps$Z)
   if(is.null(initial$mean_lam)) initial$mean_lam <- init$mean0
   
   # if(is.null(initial$scale_y)) initial$scale_y <- init$scale_y
   if(is.null(initial$scale_lam)) initial$scale_lam <- init$scale_lam
   
-  if(is.null(initial$prof_ll_lam)){
-    initial$prof_ll_lam <- TRUE
-    initial$scale_lam <- 1
-  } 
-  
-  if(initial$prof_ll_lam){
-    initial$inner <- TRUE # for Lam layer
-    initial$inner_tau2 <- TRUE
-    # if(initial$scale_lam != 1) stop("cannot specify scale and use prof ll")
-  }else{
-    initial$inner <- FALSE # for Lam layer
-    initial$inner_tau2 <- FALSE
-  }
-  
   initial$outer <- TRUE # for Y layer
   initial$tau2 <- TRUE
   
   if(is.null(initial$noise)) initial$noise = FALSE
   
   # nugget for lam layer numeric stability]
-  if (is.null(initial$g)) initial$g <- 1e-5 
+  if (is.null(initial$g)) initial$g <- 1e-6
   
-  if(is.null(initial$theta_lam)) initial$theta_lam <- init$tg[vdims]
-  if(is.null(initial$theta_y)) initial$theta_y <- init$ty
+  if(is.null(initial$theta_lam)) {
+    initial$theta_lam <- ifel(sep, init$tg, mean(init$tg))
+  }
+  if(is.null(initial$theta_y)) {
+    initial$theta_y <- ifel(sep, init$ty, mean(init$ty))
+  }
   
   if(sep){
     if(length(initial$theta_lam) != length(vdims)) 
@@ -287,10 +313,10 @@ check_inits_vdims <- function(reps, initial, n, D, sep, vdims = NULL,
   llam <- initial$llam
   # initialize lam values
   if (is.null(llam)){
-      llam <- llam_nv <- rep(0.1, nlam)
+      llam <- llam_nv <- init$llam_w # rep(0.1, nlam)
       initial$llam_nv <- llam_nv
       initial$lam_nv <- exp(llam_nv)
-      if(!is.null(reps_vdims$mult)) llam <- rep(llam, reps_vdims$mult)[reps_vdims$mult$Z] # make it len of input space and order
+      if(!is.null(reps_vdims$mult)) llam <- rep(llam, reps_vdims$mult)[reps_vdims$Z] # make it len of input space and order
     }
   
   # initial$lam_0 <- lam
@@ -309,49 +335,9 @@ check_inits_vdims <- function(reps, initial, n, D, sep, vdims = NULL,
   return(initial)
 }
 
-check_scale_vdims <- function(x, v, init, vec = FALSE, sep){
-  
-  N <- length(init$llam)
-  tau2 <- 1000
-  init$theta_lam <- init$theta_lam * 10
-  count <- 0
-  while(tau2 > 10){
-    # print(tau2)
-    init$theta_lam <- init$theta_lam/10
-    if(!vec){
-      if(sep){
-        if(v == 999) KN <- Exp2Sep(x, x, tau2 = init$scale, theta = init$theta_lam, g = init$g_0)
-        else if(v > 1000) KN <- MaternProdSep(x, x, tau2 = init$scale, theta = init$theta_lam, g = init$g_0, v= (v - 1000))
-        else KN <- MaternSep(x, x, tau2 = init$scale, theta = init$theta_lam, g = init$g_0, v = v)
-      } 
-      else{
-        if(v > 1000) v = v - 1000
-        if(v == 999) KN <- Exp2(sq_dist(x), tau2 = init$scale, theta = init$theta_lam, g = init$g_0)
-        else KN <- Matern(sq_dist(x), tau2 = init$scale, theta = init$theta_lam, g = init$g_0, v= v)
-      } 
-      Kinv <- solve(KN)
-      quadterm <- (t(init$llam_nv) %*% Kinv %*% init$llam_nv)
-      tau2 <- c(quadterm)/N
-    }else{
-      llam_ord <- init$llam_nv[x$ord]
-      U_mat <- create_U(x, init$g_0, init$theta_lam, v = v, sep = sep) / sqrt(init$scale)
-      Uty <- Matrix::crossprod(U_mat, llam_ord) # Ut * Y
-      ytUUty <- sum(Uty^2) 
-      tau2 <- c(ytUUty)/N
-    }
-    count <- count + 1
-    if(count == 10) init$g_0 <- init$g_0 * 10
-  }
-  
-  init$scale_lam <- tau2
-  if(init$theta_check == TRUE & any(init$theta_lam < init$theta_y)) init$theta_y = init$theta_lam
-  
-  return(init)
-}
-
 # add something for isotropic GP
 precalc <- function(x, out = NULL, outs2 = NULL, A = NULL, v, vec, priors, init, 
-                    latent = TRUE){
+                    latent = TRUE, vdims = FALSE){
 
   out_vec <- ifel(latent, init$llam, out)
   theta <- ifel(latent, init$theta_lam, init$theta_y)
@@ -362,6 +348,7 @@ precalc <- function(x, out = NULL, outs2 = NULL, A = NULL, v, vec, priors, init,
   b <- ifel(latent, priors$b$tau2_lam, priors$b$tau2_y)
   D <- ifel(vec, ncol(x$x_ord), ncol(x))
   
+  if(vdims & latent) out_vec <- init$llam_nv
   if(length(theta) == 1) theta <- rep(theta, D)
   
   if(!is.null(init$llam))
