@@ -40,8 +40,8 @@
 #' @param interval returns predictive variances by default \code{interval = "pi"}.
 #'        \code{interval = "ci"} returns variances for only mean process and 
 #'        \code{interval = "both"} returns both variances.
-#' @param lam_ub logical uses upper 95 quantile for latent noise to 
-#'        obtain predictive variances for the response. If \code{lam_ub = FALSE},
+#' @param lam_quan if specified uses upper \code{lam_quan} quantile for latent noise to 
+#'        obtain predictive variances for the response. If \code{lam_quan = NULL},
 #'        the mean latent noise is used for inference.
 #' @param vecchia logical uses vecchia approximation for prediction if \code{vecchia = TRUE}.
 #' @param m size of Vecchia conditioning sets (only for fits with 
@@ -115,13 +115,15 @@ NULL
 #' @rdname predict
 #' @export
 predict.bhetgp <- function(object, x_new, lite = TRUE, return_all = FALSE, 
-                           interval = c("pi", "ci", "both"), lam_ub = TRUE,
-                           cores = 1, samples = TRUE, ...){
+                           interval = c("pi", "ci", "both"),
+                           lam_quan = NULL, cores = 1, samples = TRUE, ...){
 
-  # ub_lam --> 95% uber bound of lambda for PI of y.
+  # qlam --> q% uber bound of lambda for PI of y.
   interval <- match.arg(interval)
   settings <- c(object$settings, lite = lite, return_all = return_all,
-                cores = cores, interval = interval, samples = samples, ub = lam_ub)
+                cores = cores, interval = interval, samples = samples)
+  if(is.null(lam_quan)) lam_quan <- 0.5
+  settings$qlam = lam_quan
 
   out <- list()
 
@@ -134,7 +136,7 @@ predict.bhetgp <- function(object, x_new, lite = TRUE, return_all = FALSE,
 #' @rdname predict
 #' @export
 predict.bhetgp_vec <- function(object, x_new, lite = TRUE, return_all = FALSE, 
-                               interval = c("pi", "ci", "both"), lam_ub = TRUE,
+                               interval = c("pi", "ci", "both"), lam_quan = NULL,
                                vecchia = FALSE, m = object$m, ordering_new = NULL,
                                cores = 1, omp_cores = 2, samples = TRUE, ...){
   
@@ -142,7 +144,9 @@ predict.bhetgp_vec <- function(object, x_new, lite = TRUE, return_all = FALSE,
   interval <- match.arg(interval)
   settings <- c(object$settings, lite = lite, return_all = return_all, 
                 ordering_new = ordering_new, cores = cores, interval = interval,
-                samples = samples, ub = lam_ub)
+                samples = samples)
+  if(is.null(lam_quan)) lam_quan <- 0.5
+  settings$qlam = lam_quan 
   
   # Checks OpenMP cores for Umat parallelizing
   omp_cores <- check_cores(omp_cores)
@@ -209,7 +213,7 @@ predict_nonvec <- function(object, x_new, settings){
   if (settings$cores == 1) { # run serial for loop
     
     mean_y <- mean_lam <- matrix(nrow = n_new, ncol = object$nmcmc)
-    yp_draw <- llam_draw <- matrix(nrow = n_new, ncol = object$nmcmc)
+    # yp_draw <- llam_draw <- matrix(nrow = n_new, ncol = object$nmcmc)
     if (settings$lite) {
       if(settings$interval == "pi" ||settings$interval == "both")
         s2_sum_y <- s2_sum_lam <- rep(0, times = n_new)
@@ -235,12 +239,11 @@ predict_nonvec <- function(object, x_new, settings){
                      tau2_0 = object$tau2_lam[t], sep = settings$sep, v = settings$v)
 
       mean_lam[, t] <- plam$mean
-
-      # draw from the full covariance and then sample llam
-      llam_draw[, t] <- drop(rmvnorm(1, plam$mean, diag(plam$s2, nrow = length(plam$mean))))
-
+      
+      llam_draw <- plam$mean  + qnorm(settings$qlam) * sqrt(plam$s2)
+      
       py <- gp_yw(object$y, object$x, x_new, A = object$A, lam = exp(object$llam_samples[t, ]),
-                  lam_new = exp(llam_draw[, t]), theta = object$theta_y[t, ], mu = settings$mean_y, 
+                  lam_new = exp(llam_draw), theta = object$theta_y[t, ], mu = settings$mean_y, 
                   tau2 = object$tau2[t], sep = settings$sep, v = settings$v, lite = settings$lite)
       mean_y[, t] <- py$mean
       
@@ -310,7 +313,7 @@ predict_nonvec <- function(object, x_new, settings){
         if(settings$interval == "ci" ||settings$interval == "both")
           out$sigma_sum_lam_ci <- out$sigma_sum_y_ci <- matrix(0, nrow = n_new, ncol = n_new)
       } 
-      llam_draw <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+      # llam_draw <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
       
       j <- 1
       for (t in chunks[[thread]]) {
@@ -320,11 +323,10 @@ predict_nonvec <- function(object, x_new, settings){
                        tau2_0 = object$tau2_lam[t], sep = settings$sep, v = settings$v)
 
         out$mean_lam[, j] <- plam$mean
-        llam_draw[, j] <- mvtnorm::rmvnorm(1, mean =  plam$mean, 
-                                           sigma = diag(plam$s2, nrow = length(plam$mean)))
-
+        llam_draw <- plam$mean  + qnorm(settings$qlam) * sqrt(plam$s2)
+        
         py <- gp_yw(object$y, object$x, x_new, A = object$A, lam = exp(object$llam_samples[t, ]),
-                    lam_new = exp(llam_draw[, j]), theta = object$theta_y[t, ], mu = settings$mean_y, 
+                    lam_new = exp(llam_draw), theta = object$theta_y[t, ], mu = settings$mean_y, 
                     tau2 = object$tau2[t], sep = settings$sep, v = settings$v, lite = settings$lite)
         out$mean_y[, j] <- py$mean
         
@@ -460,6 +462,7 @@ predict_nonvec_vdims <- function(object, x_new, settings, mapping = object$mappi
     xv_new <- reps_vdims_new$X0
     Av_new <- reps_vdims_new$mult
     map_new <- reps_vdims_new$Z
+    object$mappings$reps_vdims_new <- reps_vdims_new
 
     mean_np_lam <- s2_np_lam <- rep(NA, length(map_new))
     llam_n <- rep(NA, length(map))
@@ -468,21 +471,43 @@ predict_nonvec_vdims <- function(object, x_new, settings, mapping = object$mappi
     xv_new <- x_new[ , settings$vdims, drop = FALSE]
   }
 
+  nv_new <- nrow(xv_new)
+  
   if (settings$cores == 1) { # run serial for loop
 
-    mean_y <- matrix(nrow = n_new, ncol = object$nmcmc)
-    yp_draw <- matrix(nrow = n_new, ncol = object$nmcmc)
+    yp_draw <- mean_y <- matrix(nrow = n_new, ncol = object$nmcmc)
+    mean_lam <- matrix(nrow = nv_new, ncol = object$nmcmc)
+    # llam_draw <- matrix(nrow = n_new, ncol = object$nmcmc)
+      
     if (settings$lite) {
-      s2_sum_y <- rep(0, times = n_new)
-      if (settings$return_all) s2_y <- matrix(nrow = n_new, ncol = object$nmcmc)
-    } else sigma_sum_y <- matrix(0, nrow = n_new, ncol = n_new)
-    
-    mean_lam <- matrix(nrow = n_new, ncol = object$nmcmc)
-    if (settings$lite) {
-      s2_sum_lam <- rep(0, times = n_new)
-      if (settings$return_all) s2_lam <- matrix(nrow = n_new, ncol = object$nmcmc)
-    } else sigma_sum_lam <- matrix(0, nrow = n_new, ncol = n_new)
-    llam_draw <- matrix(nrow = n_new, ncol = object$nmcmc)
+      if(settings$interval == "pi" ||settings$interval == "both"){
+        s2_sum_y <- rep(0, times = n_new) 
+        s2_sum_lam <- rep(0, times = nv_new) 
+      }
+      if(settings$interval == "ci" ||settings$interval == "both"){
+        s2_sum_y_ci <- rep(0, times = n_new)
+        s2_sum_lam_ci <- rep(0, times = nv_new)
+      }
+      if (settings$return_all){
+        if(settings$interval == "pi" ||settings$interval == "both"){
+          s2_y <- matrix(nrow = n_new, ncol = object$nmcmc)
+          s2_lam <- matrix(nrow = nv_new, ncol = object$nmcmc)
+        }
+        if(settings$interval == "ci" ||settings$interval == "both"){
+          s2_y_ci <- matrix(nrow = n_new, ncol = object$nmcmc)
+          s2_lam_ci <- matrix(nrow = nv_new, ncol = object$nmcmc) 
+        }
+      } 
+    } else{
+      if(settings$interval == "pi" ||settings$interval == "both"){
+        sigma_sum_y <- matrix(0, nrow = n_new, ncol = n_new)
+        sigma_sum_lam <- matrix(0, nrow = nv_new, ncol = nv_new)
+      }
+      if(settings$interval == "ci" ||settings$interval == "both"){
+        sigma_sum_y_ci <- matrix(0, nrow = n_new, ncol = n_new) 
+        sigma_sum_lam_ci <- matrix(0, nrow = nv_new, ncol = nv_new) 
+      }
+    } 
 
     for (t in 1:object$nmcmc) {
 
@@ -494,33 +519,53 @@ predict_nonvec_vdims <- function(object, x_new, settings, mapping = object$mappi
       if(!is.null(mapping$reps_vdims)){
         mean_np_lam[map_new] <- rep(plam$mean, Av_new)
         s2_np_lam[map_new] <- rep(plam$s2, Av_new)
-        llam_draw_npv <- rmvnorm(1, mean =  plam$mean, sigma = plam$sigma, checkSymmetry = FALSE)
-        llam_draw[map_new, t] <- rep(llam_draw_npv, Av_new)
+        llam_draw <- mean_np_lam + qnorm(settings$qlam) * sqrt(s2_np_lam)
+        
         llam_n[map] <- rep(object$llam_samples[t, ], Av)
       }else{
         mean_np_lam <- plam$mean
         s2_np_lam <- plam$s2
-        llam_draw[, t] <-  rmvnorm(1, mean =  plam$mean, sigma = plam$sigma, checkSymmetry = FALSE)
+        llam_draw <- plam$mean  + qnorm(settings$qlam) * sqrt(plam$s2)
         llam_n <- object$llam_samples[t, ]
       }
-
-      mean_lam[, t] <- mean_np_lam
-
-      if (settings$lite) { # This should not be plam$s2 -> that's unordered
-        s2_sum_lam <- s2_sum_lam + plam$s2
-        if (settings$return_all) s2_lam[, t] <- plam$s2
-      } else sigma_sum_lam <- sigma_sum_lam + plam$sigma
+      
+      mean_lam[, t] <- plam$mean
       
       py <- gp_yw(object$y, object$x, x_new, A = object$A, lam = exp(llam_n), 
-                  lam_new = exp(llam_draw[, t]), theta = object$theta_y[t, ], mu = settings$mean_y, 
+                  lam_new = exp(llam_draw), theta = object$theta_y[t, ], mu = settings$mean_y, 
                   tau2 =object$tau2[t], sep = settings$sep, v = settings$v, lite = settings$lite)
       
       mean_y[, t] <- py$mean
-      if (settings$lite) {
-        s2_sum_y <- s2_sum_y + py$s2
-        if (settings$return_all) s2_y[, t] <- py$s2 # storing individual variances for each posterior sample
-      } else sigma_sum_y <- sigma_sum_y + py$sigma # storing main covariance matrix
       
+      if (settings$lite) {
+        if(settings$interval == "pi" ||settings$interval == "both"){
+          s2_sum_lam <- s2_sum_lam + plam$s2
+          s2_sum_y <- s2_sum_y + py$s2
+        }
+        if(settings$interval == "ci" ||settings$interval == "both"){
+          s2_sum_lam_ci <- s2_sum_lam_ci + plam$s2_ci
+          s2_sum_y_ci <- s2_sum_y_ci + py$s2_ci
+        }
+        if (settings$return_all){
+          if(settings$interval == "pi" ||settings$interval == "both"){
+            s2_lam[, t] <- plam$s2
+            s2_y[, j] <- py$s2  
+          }
+          if(settings$interval == "ci" ||settings$interval == "both"){
+            s2_lam_ci[, t] <- plam$s2_ci
+            s2_y_ci[, j] <- py$s2_ci
+          }
+        } 
+      } else{
+        if(settings$interval == "pi" ||settings$interval == "both"){
+          sigma_sum_lam <- sigma_sum_lam + plam$sigma
+          sigma_sum_y <- sigma_sum_y + py$sigma # storing main covariance matrix 
+        }
+        if(settings$interval == "ci" ||settings$interval == "both"){
+          sigma_sum_lam_ci <- sigma_sum_lam_ci + plam$sigma_ci
+          sigma_sum_y_ci <- sigma_sum_y_ci + py$sigma_ci # storing main covariance matrix
+        }
+      } 
       # yp_draw[, t] <- rmvnorm(1, mean = mean_y[, t], sigma = diag (py$s2))
     } # end of t for loop
 
@@ -540,20 +585,40 @@ predict_nonvec_vdims <- function(object, x_new, settings, mapping = object$mappi
     result <- foreach(thread = 1:settings$cores) %dopar% {
       out <- list()
       out$mean_y <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
-      out$mean_lam <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+      out$mean_lam <- matrix(nrow = nv_new, ncol = length(chunks[[thread]]))
       if (settings$lite) {
-        out$s2_sum_y <- rep(0, times = n_new)
-        out$s2_sum_lam <- rep(0, times = n_new)
+        if(settings$interval == "pi" ||settings$interval == "both"){
+          out$s2_sum_y <- rep(0, times = nv_new)
+          out$s2_sum_lam <- rep(0, times = nv_new) 
+        }
+        if(settings$interval == "ci" ||settings$interval == "both"){
+          out$s2_sum_y_ci <- rep(0, times = nv_new)
+          out$s2_sum_lam_ci <- rep(0, times = nv_new)
+        }
         if (settings$return_all) {
-          out$s2_y <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
-          out$s2_lam <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+          if(settings$interval == "pi" ||settings$interval == "both"){
+            out$s2_y <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+            out$s2_lam <- matrix(nrow = nv_new, ncol = length(chunks[[thread]]))
+          }
+          if(settings$interval == "ci" ||settings$interval == "both"){
+            out$s2_y_ci <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+            out$s2_lam_ci <- matrix(nrow = nv_new, ncol = length(chunks[[thread]]))
+          }
         }
       } else {
-        out$sigma_sum_y <- matrix(0, nrow = n_new, ncol = n_new)
-        out$sigma_sum_lam <- matrix(0, nrow = n_new, ncol = n_new)
+        if(settings$interval == "pi" ||settings$interval == "both"){
+          out$sigma_sum_y <- matrix(0, nrow = n_new, ncol = n_new)
+          out$sigma_sum_lam <- matrix(0, nrow = nv_new, ncol = nv_new)
+        }
+        if(settings$interval == "ci" ||settings$interval == "both"){
+          out$sigma_sum_y_ci <- matrix(0, nrow = n_new, ncol = n_new)
+          out$sigma_sum_lam_ci <- matrix(0, nrow = nv_new, ncol = nv_new) 
+        }
       }
 
-      llam_draw <- yp_draw <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+      yp_draw <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+      # llam_draw <- matrix(nrow = n_new, ncol = length(chunks[[thread]]))
+      llam_draw <- rep(NA, n_new)
       j <- 1
       for (t in chunks[[thread]]) {
         
@@ -565,36 +630,55 @@ predict_nonvec_vdims <- function(object, x_new, settings, mapping = object$mappi
         if(!is.null(mapping$reps_vdims)){
           mean_np_lam[map_new] <- rep(plam$mean, Av_new)
           s2_np_lam[map_new] <- rep(plam$s2, Av_new)
-          llam_draw_npv <- rmvnorm(1, mean =  plam$mean, sigma = plam$sigma, checkSymmetry = FALSE)
-          llam_draw[map_new, j] <- rep(llam_draw_npv, Av_new)
+          llam_draw <- mean_np_lam + qnorm(settings$qlam) * sqrt(s2_np_lam)
           llam_n[map] <- rep(object$llam_samples[t, ], Av)
         }else{
           mean_np_lam <- plam$mean
           s2_np_lam <- plam$s2
-          llam_draw[, j] <-  rmvnorm(1, mean =  plam$mean, sigma = plam$sigma, checkSymmetry = FALSE)
+          llam_draw <- plam$mean  + qnorm(settings$qlam) * sqrt(plam$s2)
           llam_n <- object$llam_samples[t, ]
         }
-        
-        mean_lam[, j] <- mean_np_lam
-        
-        if (settings$lite) { # This should not be plam$s2 -> that's unordered
-          s2_sum_lam <- s2_sum_lam + plam$s2
-          if (settings$return_all) s2_lam[, t] <- plam$s2
-        } else sigma_sum_lam <- sigma_sum_lam + plam$sigma
+        out$mean_lam[, j] <- plam$mean
         
         py <- gp_yw(object$y, object$x, x_new, A = object$A, lam = exp(llam_n), 
-                    lam_new = exp(llam_draw[, j]), theta = object$theta_y[t, ], mu = settings$mean_y, 
+                    lam_new = exp(llam_draw), theta = object$theta_y[t, ], mu = settings$mean_y, 
                     tau2 =object$tau2[t], sep = settings$sep, v = settings$v, lite = settings$lite)
         
-        mean_y[, j] <- py$mean
+        out$mean_y[, j] <- py$mean
+        
         if (settings$lite) {
-          s2_sum_y <- s2_sum_y + py$s2
-          if (settings$return_all) s2_y[, j] <- py$s2 # storing individual variances for each posterior sample
-        } else sigma_sum_y <- sigma_sum_y + py$sigma # storing main covariance matrix
+          if(settings$interval == "pi" ||settings$interval == "both"){
+            out$s2_sum_lam <- out$s2_sum_lam + plam$s2
+            out$s2_sum_y <- out$s2_sum_y + py$s2
+          }
+          if(settings$interval == "ci" ||settings$interval == "both"){
+            out$s2_sum_lam_ci <- out$s2_sum_lam_ci + plam$s2_ci
+            out$s2_sum_y_ci <- out$s2_sum_y_ci + py$s2_ci
+          }
+          if (settings$return_all){
+            if(settings$interval == "pi" ||settings$interval == "both"){
+              out$s2_lam[, j] <- plam$s2
+              out$s2_y[, j] <- py$s2  
+            }
+            if(settings$interval == "ci" ||settings$interval == "both"){
+              out$s2_lam_ci[, j] <- plam$s2_ci
+              out$s2_y_ci[, j] <- py$s2_ci
+            }
+          } 
+        } else{
+          if(settings$interval == "pi" ||settings$interval == "both"){
+            out$sigma_sum_lam <- out$sigma_sum_lam + plam$sigma
+            out$sigma_sum_y <- out$sigma_sum_y + py$sigma # storing main covariance matrix 
+          }
+          if(settings$interval == "ci" ||settings$interval == "both"){
+            out$sigma_sum_lam_ci <- out$sigma_sum_lam + plam$sigma_ci
+            out$sigma_sum_y_ci <- out$sigma_sum_y + py$sigma_ci # storing main covariance matrix
+          }
+        } 
         
-        # yp_draw[, j] <- rmvnorm(1, mean = py$mean, sigma = diag (py$s2))
         j <- j + 1
       } # end of t for loop
+      return(out)
     } # end of foreach loop
 
     stopCluster(cl)
@@ -604,37 +688,72 @@ predict_nonvec_vdims <- function(object, x_new, settings, mapping = object$mappi
     mean_lam <- do.call(cbind, lapply(result, with, eval(parse(text = "mean_lam"))))
 
     if (settings$lite) {
-      s2_sum_y <- Reduce("+", lapply(result, with, eval(parse(text = "s2_sum_y"))))
-      s2_sum_lam <- Reduce("+", lapply(result, with, eval(parse(text = "s2_sum_lam"))))
+      if(settings$interval == "pi" ||settings$interval == "both"){
+        s2_sum_y <- Reduce("+", lapply(result, with, eval(parse(text = "s2_sum_y"))))
+        s2_sum_lam <- Reduce("+", lapply(result, with, eval(parse(text = "s2_sum_lam")))) 
+      }
+      if(settings$interval == "ci" ||settings$interval == "both"){
+        s2_sum_y_ci <- Reduce("+", lapply(result, with, eval(parse(text = "s2_sum_y_ci"))))
+        s2_sum_lam_ci <- Reduce("+", lapply(result, with, eval(parse(text = "s2_sum_lam_ci"))))
+      }
       if (settings$return_all) {
-        s2_y <- do.call(cbind, lapply(result, with, eval(parse(text = "s2_y"))))
-        s2_lam <- do.call(cbind, lapply(result, with, eval(parse(text = "s2_lam"))))
+        if(settings$interval == "pi" ||settings$interval == "both"){
+          s2_y <- do.call(cbind, lapply(result, with, eval(parse(text = "s2_y"))))
+          s2_lam <- do.call(cbind, lapply(result, with, eval(parse(text = "s2_lam")))) 
+        }
+        if(settings$interval == "ci" ||settings$interval == "both"){
+          s2_y_ci <- do.call(cbind, lapply(result, with, eval(parse(text = "s2_y_ci"))))
+          s2_lam_ci <- do.call(cbind, lapply(result, with, eval(parse(text = "s2_lam_ci"))))
+        }
       }
     } else {
-      sigma_sum_y <- Reduce("+", lapply(result, with, eval(parse(text = "sigma_sum_y"))))
-      sigma_sum_lam <- Reduce("+", lapply(result, with, eval(parse(text = "sigma_sum_lam"))))
+      if(settings$interval == "pi" ||settings$interval == "both"){
+        sigma_sum_y <- Reduce("+", lapply(result, with, eval(parse(text = "sigma_sum_y"))))
+        sigma_sum_lam <- Reduce("+", lapply(result, with, eval(parse(text = "sigma_sum_lam"))))
+      }
+      if(settings$interval == "ci" ||settings$interval == "both"){
+        sigma_sum_y_ci <- Reduce("+", lapply(result, with, eval(parse(text = "sigma_sum_y_ci"))))
+        sigma_sum_lam_ci <- Reduce("+", lapply(result, with, eval(parse(text = "sigma_sum_lam_ci"))))
+      }
     }
+    
   } # end of else statement
 
   # Add variables to the output list
   object$mean <- rowMeans(mean_y)
-  if (settings$lite) {
-    object$s2_y <- s2_sum_y / object$nmcmc + apply(mean_y, 1, var)
-    if (settings$return_all) {
-      object$mean_all <- mean_y
-      object$s2_all <- s2_y
-    }
-  } else object$Sigma <- sigma_sum_y / object$nmcmc + cov(t(mean_y))
-  
   object$mean_lnugs <- rowMeans(mean_lam)
   if (settings$lite) {
-    object$s2_lnugs <- s2_sum_lam / object$nmcmc + apply(mean_lam, 1, var)
-    if (settings$return_all) {
-      object$mean_lnugs_all <- mean_lam
-      object$s2_lnugs_all <- s2_lam
+    if(settings$interval == "pi" ||settings$interval == "both"){
+      object$s2_y <- s2_sum_y / object$nmcmc + apply(mean_y, 1, var)
+      object$s2_lnugs <- s2_sum_lam / object$nmcmc + apply(mean_lam, 1, var) 
     }
-  } else object$Sigma_nugs <- sigma_sum_lam / object$nmcmc + cov(t(mean_lam))
-
+    if(settings$interval == "ci" ||settings$interval == "both"){
+      object$s2_y_ci <- s2_sum_y_ci / object$nmcmc + apply(mean_y, 1, var)
+      object$s2_lnugs_ci <- s2_sum_lam_ci / object$nmcmc + apply(mean_lam, 1, var)
+    }
+    if (settings$return_all) {
+      object$mean_all <- mean_y
+      object$mean_lnugs_all <- mean_lam
+      if(settings$interval == "pi" ||settings$interval == "both"){
+        object$s2_all <- s2_y
+        object$s2_lnugs_all <- s2_lam
+      }
+      if(settings$interval == "ci" ||settings$interval == "both"){
+        object$s2_all_ci <- s2_y_ci
+        object$s2_lnugs_all_ci <- s2_lam_ci
+      }
+    }
+  } else {
+    if(settings$interval == "pi" ||settings$interval == "both"){
+      object$Sigma <- sigma_sum_y / object$nmcmc + cov(t(mean_y))
+      object$Sigma_nugs <- sigma_sum_lam / object$nmcmc + cov(t(mean_lam)) 
+    }
+    if(settings$interval == "ci" ||settings$interval == "both"){
+      object$Sigma_ci <- sigma_sum_y_ci / object$nmcmc + cov(t(mean_y))
+      object$Sigma_nugs_ci <- sigma_sum_lam_ci / object$nmcmc + cov(t(mean_lam))
+    }
+  }
+  
   toc <- proc.time()[[3]]
   object$time <- object$time + unname(toc - tic)
 
